// =================================================================================================
//
//   File....... p2_to_processing.pde
//   Purpose.... Accepts command bytes via serial stream
//   Author..... Jon "JonnyMac" McPhalen
//               Copyright (c) 2020 Jon McPhalen
//               -- see below for terms of use
//   E-mail..... jon.mcphalen@gmail.com
//   Started....
//   Updated.... 30 DEC 2020
//
// =================================================================================================


final int R_WIDTH  = 80;                                        // robot width  (horizontal)
final int R_HEIGHT = 30;                                        // robot height (horizontal)
final int R_DELTA  = 20;                                        // position change per encoder click

final int EW = 0;                                               // orientations
final int NS = 1;


import processing.serial.*;                                     // allow serial

int posx;                                                       // robot position
int posy;
int orientation = EW;                                           // initial orientation

Serial port;                                                    // create a serial port


void setup()
{
  size(800, 600);                                               // set window size & title
  noStroke();
  surface.setTitle("P2 Serial to Processing");

  posx = width/2;                                               // center the robot
  posy = height/2;

  showRobot();                                                  // display

  port = new Serial(this, "COM1", 230400);                      // set your serial port here
}


void draw()
{
  if (port.available() > 0) {                                   // have command byte?
    int c = port.read();                                        // yes, get it
    // println(c);

    if (c == '*') {                                             // orientation change?
      orientation = (orientation == EW) ? NS : EW;
    }
    else if (c == '+') {                                        // forward
      if (orientation == EW) {
        if (posx < width-(R_WIDTH/2+R_DELTA)) {
          posx += R_DELTA;
        }
      }
      else {
        if (posy < height-(R_WIDTH/2+R_DELTA)) {
          posy += R_DELTA;
        }
      }
    }
    else if (c == '-') {                                        // reverse
      if (orientation == EW) {
        if (posx > (R_WIDTH/2+R_DELTA)) {
          posx -= R_DELTA;
        }
      }
      else {
        if (posy > (R_WIDTH/2+R_DELTA)) {
          posy -= R_DELTA;
        }
      }
    }
    showRobot();
  }
}


void showRobot()
{
  background(220);                                              // earase last
  fill(64, 64, 255);                                            // blue
  if (orientation == EW) {
    // horizontal
    rect(posx-(R_WIDTH/2), posy-(R_HEIGHT/2), R_WIDTH, R_HEIGHT);
  }
  else {
    // vertical
    rect(posx-(R_HEIGHT/2), posy-(R_WIDTH/2), R_HEIGHT, R_WIDTH);
  }
}
